/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_IO_EFFICIENCY_TEST_H
#define CUBELIB_IO_EFFICIENCY_TEST_H

#include <string>
#include "PerformanceTest.h"
#include "POPPosixIOTimeTest.h"
#include "POPMpiIOTimeTest.h"

namespace popcalculation
{
class POPIOEfficiencyTest : public PerformanceTest
{
private:
    POPPosixIOTime* posix_io;
    POPMpiIOTime*   mpi_io;

    void
    calculate();


protected:

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );


    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "IO Efficiency is an additional metric, indicating amount of time spend not in a useful computation, \n"
            "but in the processing of IO operations. \n"
            "IO Efficiency can be split into two components:\n"
            "\tPOSIX IO Efficiency shows the fraction of time spent in POSIX IO calls\n"
            "\tMPI IO Efficiency shows the fraction of time spent in MPI IO calls\n"
            "In this analysis IO an be computed as a sum of these two sub-metrics minus one:\n"
            "\tIO = 1 - POSIX IO + MPI IO  \n";
        std::string not_active_text =
            "IO Efficiency  metric is available for such Score-P/Scalasca measurement, where MPI or POSIX IO calls are present. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time and hence IO Time .\n"
            "In this case this metric reported as NAN.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;


public:
    POPIOEfficiencyTest( POPPosixIOTime*,
                         POPMpiIOTime* );

    virtual
    ~POPIOEfficiencyTest()
    {
    };

    bool
    isActive() const;

    bool
    isIssue() const;


    virtual
    std::list<PerformanceTest*>
    getPrereqs()
    {
        std::list<PerformanceTest*> prereqs;
        if ( posix_io != nullptr && mpi_io != nullptr )
        {
            prereqs.push_back( posix_io );
            prereqs.push_back( mpi_io );
        }
        return prereqs;
    }
};
};
#endif // ADVISER_RATING_WIDGET_H
